@@ -18,9 +18,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: dummy.c,v 1.3 2008-06-09 10:32:00 hanke Exp $
  */
@@ -39,7 +37,7 @@
 #define MODULE_NAME     "dummy"
 #define MODULE_VERSION  "0.1"
 
-//#define Debug 0
+DECLARE_DEBUG()
 
 /* Thread and process control */
 static int dummy_speaking = 0;
@@ -47,6 +45,7 @@ static int dummy_speaking = 0;
 static pthread_t dummy_speak_thread;
 static pid_t dummy_pid;
 static sem_t dummy_semaphore;
+static gboolean initialized = FALSE;
 
 /* Internal functions prototypes */
 static void *_dummy_speak(void *);
@@ -60,6 +59,8 @@ int module_load(void)
 
 	INIT_SETTINGS_TABLES();
 
+	REGISTER_DEBUG();
+
 	return 0;
 }
 
@@ -73,7 +74,7 @@ int module_init(char **status_info)
 
 	DBG("Dummy: creating new thread for dummy_speak\n");
 	dummy_speaking = 0;
-	ret = pthread_create(&dummy_speak_thread, NULL, _dummy_speak, NULL);
+	ret = spd_pthread_create(&dummy_speak_thread, NULL, _dummy_speak, NULL);
 	if (ret != 0) {
 		DBG("Dummy: thread failed\n");
 		*status_info = g_strdup("The module couldn't initialize threads"
@@ -83,6 +84,7 @@ int module_init(char **status_info)
 		return -1;
 	}
 
+	initialized = TRUE;
 	*status_info = g_strdup("Everything ok so far.");
 
 	DBG("Ok, now debugging");
@@ -92,7 +94,12 @@ int module_init(char **status_info)
 
 SPDVoice **module_list_voices(void)
 {
-	return NULL;
+	static SPDVoice voice = {
+		.name = "dummy",
+		.language = "en",
+	};
+	static SPDVoice *voices[] = { &voice, NULL };
+	return voices;
 }
 
 int module_speak(gchar * data, size_t bytes, SPDMessageType msgtype)
@@ -111,7 +118,7 @@ int module_speak(gchar * data, size_t bytes, SPDMessageType msgtype)
 	dummy_speaking = 1;
 	sem_post(&dummy_semaphore);
 
-	DBG("Dummy: leaving write() normaly\n\r");
+	DBG("Dummy: leaving write() normally\n\r");
 	return bytes;
 }
 
@@ -148,6 +155,9 @@ int module_close(void)
 {
 	DBG("dummy: close()\n");
 
+	if (!initialized)
+		return 0;
+
 	if (dummy_speaking) {
 		module_stop();
 	}
@@ -157,6 +167,7 @@ int module_close(void)
 
 	sem_destroy(&dummy_semaphore);
 
+	initialized = FALSE;
 	return 0;
 }
 
@@ -168,6 +179,7 @@ void *_dummy_speak(void *nothing)
 
 	DBG("dummy: speaking thread starting.......\n");
 
+	/* Make interruptible */
 	set_speaking_thread_parameters();
 
 	while (1) {
@@ -243,7 +255,7 @@ void _dummy_child()
 	    g_strdup("aplay  " DATADIR
 		     "/dummy-message.wav > /dev/null 2> /dev/null");
 	try3 =
-	    g_strdup("paplay " DATADIR
+	    g_strdup("paplay -n speech-dispatcher " DATADIR
 		     "/dummy-message.wav > /dev/null 2> /dev/null");
 
 	DBG("child: synth commands = |%s|%s|%s|", try1, try2, try3);
@@ -253,7 +265,7 @@ void _dummy_child()
 	ret = system(try1);
 	DBG("Executed shell command '%s' returned with %d", try1, ret);
 	if ((ret != 0)) {
-		DBG("Execution failed, trying seccond command");
+		DBG("Execution failed, trying second command");
 		ret = system(try2);
 		DBG("Executed shell command '%s' returned with %d", try1, ret);
 		if ((ret != 0)) {
