@@ -137,8 +137,8 @@ DECLARE_DEBUG();
 	{ \
 		T ## name *new_item; \
 		char* new_key; \
-		new_item = (T ## name *) g_malloc(sizeof(T ## name)); \
 		if (cmd->data.list[0] == NULL) return NULL; \
+		new_item = (T ## name *) g_malloc(sizeof(T ## name)); \
 		new_key = g_strdup(cmd->data.list[0]); \
 		new_item->arg1 = (int) strtol(cmd->data.list[1], NULL, 10); \
 		new_item->arg2 = (int) strtol(cmd->data.list[2], NULL, 10); \
@@ -1526,7 +1526,7 @@ gboolean alloc_voice_list()
 	int nLanguages = MAX_NB_OF_LANGUAGES;
 	int i = 0;
 
-	if (eciGetAvailableLanguages(aLanguage, &nLanguages))
+	if (eciGetAvailableLanguages(aLanguage, &nLanguages) || nLanguages == 0)
 		return FALSE;
 
 	speechd_voice = g_malloc((nLanguages + 1) * sizeof(SPDVoice *));
@@ -1678,6 +1678,7 @@ static void load_user_dictionary()
 				DBG(DBG_MODNAME "%s is not a directory",
 				    dirname->str);
 				g_free(language);
+				g_string_free(dirname, TRUE);
 				return;
 			}
 		}
