@@ -15,9 +15,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: module.c,v 1.40 2008-07-07 14:30:51 hanke Exp $
  */
@@ -29,24 +27,233 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
+#include <netinet/in.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <dirent.h>
 #include <glib.h>
+#include <dotconf.h>
 
 #include "speechd.h"
-#include <spd_utils.h>
 #include "output.h"
 #include "module.h"
 
+static char *spd_get_path(const char *filename, const char *startdir)
+{
+	char *ret;
+	if (filename == NULL)
+		return NULL;
+	if (filename[0] != '/') {
+		if (startdir == NULL)
+			ret = g_strdup(filename);
+		else
+			ret = g_strdup_printf("%s/%s", startdir, filename);
+	} else {
+		ret = g_strdup(filename);
+	}
+	return ret;
+}
+
 void destroy_module(OutputModule * module)
 {
+	close(module->pipe_speak[0]);
+	close(module->pipe_speak[1]);
+	if (module->stderr_redirect >= 0)
+		close(module->stderr_redirect);
 	g_free(module->name);
 	g_free(module->filename);
 	g_free(module->configfilename);
+	g_free(module->debugfilename);
+	g_free(module->progdir);
+	g_free(module->configdir);
 	g_free(module);
 }
 
+/*
+ * Check that we can execute the configured command
+ */
+DOTCONF_CB(GenericCmdDependency_cb)
+{
+	unsigned *missing_dep = ctx;
+	char s[5 + strlen(cmd->data.str) + 17 + 1];
+
+	if (!cmd->data.str[0])
+		return NULL;
+
+	snprintf(s, sizeof(s), "type %s > /dev/null 2>&1", cmd->data.str);
+	if (system(s) != 0)
+	{
+		MSG(5, "Did not find command %s", cmd->data.str);
+		(*missing_dep)++;
+	}
+	return NULL;
+}
+
+/*
+ * Check that we can connect to the configured local port
+ */
+DOTCONF_CB(GenericPortDependency_cb)
+{
+	unsigned *missing_dep = ctx;
+	int s = socket(PF_INET, SOCK_STREAM, 0);
+	struct sockaddr_in sin;
+
+	if (s < 0)
+	{
+		MSG(5, "Could not establish IPv4 socket: %s", strerror(errno));
+		(*missing_dep)++;
+		return NULL;
+	}
+
+	memset(&sin, 0, sizeof(sin));
+	sin.sin_family = AF_INET;
+	sin.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+	sin.sin_port = htons(cmd->data.value);
+
+	if (connect(s, (struct sockaddr *) &sin, sizeof(sin)) < 0)
+	{
+		MSG(5, "Could not connect to IPv4 socket: %s", strerror(errno));
+		(*missing_dep)++;
+	}
+	close(s);
+
+	return NULL;
+}
+
+FUNC_ERRORHANDLER(ignore_errors)
+{
+	return 0;
+}
+
+#define FNAME_PREFIX "sd_"
+static GList *detect_generic_modules(GList *modules, const char *dirname, const char *config_dirname)
+{
+	struct dirent *entry;
+	char **module_parameters;
+	char *full_path;
+	struct stat fileinfo;
+	int sys_ret;
+
+	/* Special-case the generic module: autoload for the various
+	 * configurations */
+	full_path = spd_get_path("modules", config_dirname);
+	MSG(5, "Looking for generic variants configurations in %s",
+	    full_path);
+	DIR *config_dir = opendir(full_path);
+	if (config_dir == NULL)
+	{
+		MSG(4, "couldn't open directory %s because of error %s\n",
+		    full_path, strerror(errno));
+		g_free(full_path);
+		return modules;
+	}
+
+	while (NULL != (entry = readdir(config_dir))) {
+		size_t len;
+		char *file_path;
+
+		static const configoption_t options[] = {
+			{
+				.name = "GenericCmdDependency",
+				.type = ARG_STR,
+				.callback = GenericCmdDependency_cb,
+				.info = NULL,
+				.context = 0,
+			},
+			{
+				.name = "GenericPortDependency",
+				.type = ARG_INT,
+				.callback = GenericPortDependency_cb,
+				.info = NULL,
+				.context = 0,
+			},
+			{
+				.name = "",
+				.type = 0,
+				.callback = NULL,
+				.info = NULL,
+				.context = 0,
+			}
+		};
+		configfile_t *configfile;
+		unsigned missing_dep = 0;
+
+		if (!strcmp(entry->d_name, ".") || !strcmp(entry->d_name, ".."))
+			continue;
+		file_path = spd_get_path(entry->d_name, full_path);
+		sys_ret = stat(file_path, &fileinfo);
+		if (sys_ret != 0) {
+			MSG(4, "stat failed on file %s in %s", entry->d_name,
+			    full_path);
+			g_free(file_path);
+			continue;
+		}
+
+		/* Note: stat(2) dereferences symlinks. */
+		if (!S_ISREG(fileinfo.st_mode)) {
+			MSG(4, "Ignoring %s in %s; not a regular file.",
+			    entry->d_name, full_path);
+			g_free(file_path);
+			continue;
+		}
+
+		len = strlen(entry->d_name);
+		if (strcmp(entry->d_name + len - strlen("-generic.conf"),
+			   "-generic.conf")) {
+			MSG(5, "Ignoring %s: not named something-generic.conf",
+			    entry->d_name);
+			g_free(file_path);
+			continue;
+		}
+
+		/* Check for actual binaries and ports given by GenericCmdDependency and GenericPortDependency */
+
+		configfile = dotconf_create(file_path, options,
+					    &missing_dep, CASE_INSENSITIVE);
+		if (!configfile) {
+			MSG(5, "Ignoring %s: Can not parse config file", file_path);
+			g_free(file_path);
+			continue;
+		}
+		configfile->errorhandler = (dotconf_errorhandler_t) ignore_errors;
+
+		if (dotconf_command_loop(configfile) == 0) {
+			MSG(5, "Ignoring %s: Can not parse config file", file_path);
+			g_free(file_path);
+			dotconf_cleanup(configfile);
+			continue;
+		}
+		dotconf_cleanup(configfile);
+
+		if (missing_dep != 0) {
+			MSG(5, "Ignoring %s: did not find %d dependency",
+			       file_path, missing_dep);
+			g_free(file_path);
+			continue;
+		}
+		g_free(file_path);
+
+		module_parameters = g_malloc(6 * sizeof(char *));
+		module_parameters[1] = g_strdup(FNAME_PREFIX "generic");
+		module_parameters[2] = g_strdup(entry->d_name);
+		entry->d_name[len - strlen(".conf")] = '\0';
+		module_parameters[0] = g_strdup(entry->d_name);
+		module_parameters[3] =
+		    g_strdup_printf("%s/%s.log", SpeechdOptions.log_dir,
+				    entry->d_name);
+		module_parameters[4] = g_strdup(dirname);
+		module_parameters[5] = g_strdup(full_path);
+		modules = g_list_append(modules, module_parameters);
+
+		MSG(5,
+		    "Module name=%s being inserted into detected_modules list",
+		    entry->d_name);
+	}
+	g_free(full_path);
+	closedir(config_dir);
+	return modules;
+}
+
 /*
  * detect_output_modules: automatically discover all available modules.
  * Parameters:
@@ -55,21 +262,23 @@ void destroy_module(OutputModule * module)
  * For each file in the directory containing module binaries,
  * add an entry to a list of discovered modules.
  */
-GList *detect_output_modules(char *dirname)
+GList *detect_output_modules(GList *modules, const char *dirname, const char *user_config_dirname, const char *config_dirname)
 {
-	static const int FNAME_PREFIX_LENGTH = 3;
+	static const int FNAME_PREFIX_LENGTH = strlen(FNAME_PREFIX);
 	DIR *module_dir = opendir(dirname);
 	struct dirent *entry;
+	char *module_name;
 	char **module_parameters;
-	GList *modules = NULL;
 	char *full_path;
 	struct stat fileinfo;
 	int sys_ret;
 
+	MSG(5, "Detecting modules from %s\n", dirname);
+
 	if (module_dir == NULL) {
 		MSG(3, "couldn't open directory %s because of error %s\n",
 		    dirname, strerror(errno));
-		return NULL;
+		return modules;
 	}
 
 	while (NULL != (entry = readdir(module_dir))) {
@@ -91,7 +300,7 @@ GList *detect_output_modules(char *dirname)
 			continue;
 		}
 
-		if (strncmp(entry->d_name, "sd_", FNAME_PREFIX_LENGTH)
+		if (strncmp(entry->d_name, FNAME_PREFIX, FNAME_PREFIX_LENGTH)
 		    || (entry->d_name[FNAME_PREFIX_LENGTH] == '\0')) {
 			MSG(1,
 			    "Module discovery ignoring %s: malformed filename.",
@@ -99,28 +308,36 @@ GList *detect_output_modules(char *dirname)
 			continue;
 		}
 
-		module_parameters = g_malloc(4 * sizeof(char *));
-		module_parameters[0] =
-		    g_strdup(entry->d_name + FNAME_PREFIX_LENGTH);
-		module_parameters[1] = g_strdup(entry->d_name);
-		module_parameters[2] =
-		    g_strdup_printf("%s.conf", module_parameters[0]);
-		module_parameters[3] =
-		    g_strdup_printf("%s/%s.log", SpeechdOptions.log_dir,
-				    module_parameters[0]);
-		modules = g_list_append(modules, module_parameters);
+		module_name = entry->d_name + FNAME_PREFIX_LENGTH;
+		if (strcmp(module_name, "generic") != 0) {
+			module_parameters = g_malloc(6 * sizeof(char *));
+			module_parameters[0] = g_strdup(module_name);
+			module_parameters[1] = g_strdup(entry->d_name);
+			module_parameters[2] =
+			    g_strdup_printf("%s.conf", module_parameters[0]);
+			module_parameters[3] =
+			    g_strdup_printf("%s/%s.log", SpeechdOptions.log_dir,
+					    module_parameters[0]);
+			module_parameters[4] = g_strdup(dirname);
+			module_parameters[5] = NULL;
+			modules = g_list_append(modules, module_parameters);
 
-		MSG(5,
-		    "Module name=%s being inserted into detected_modules list",
-		    module_parameters[0]);
+			MSG(5,
+			    "Module name=%s being inserted into detected_modules list",
+			    module_parameters[0]);
+		} else {
+			modules = detect_generic_modules(modules, dirname, user_config_dirname);
+			modules = detect_generic_modules(modules, dirname, config_dirname);
+		}
 	}
 
 	closedir(module_dir);
 	return modules;
 }
 
-OutputModule *load_output_module(char *mod_name, char *mod_prog,
-				 char *mod_cfgfile, char *mod_dbgfile)
+OutputModule *load_output_module(const char *mod_name, const char *mod_prog,
+				 const char *mod_cfgfile, const char *mod_dbgfile,
+				 const char *mod_prog_dir, const char *mod_cfg_dir)
 {
 	OutputModule *module;
 	int fr;
@@ -132,21 +349,52 @@ OutputModule *load_output_module(char *mod_name, char *mod_prog,
 	size_t n = 0;
 	char s;
 	GString *reply;
+	struct stat fileinfo;
+	int i;
 
 	if (mod_name == NULL)
 		return NULL;
 
 	module = (OutputModule *) g_malloc(sizeof(OutputModule));
+	if (!module)
+		return NULL;
+	if (pipe(module->pipe_speak) != 0) {
+		g_free(module);
+		return NULL;
+	}
 
 	module->name = (char *)g_strdup(mod_name);
-	module->filename = (char *)spd_get_path(mod_prog, SpeechdOptions.module_dir);
+	module->progdir = g_strdup(mod_prog_dir);
+	module->configdir = g_strdup(mod_cfg_dir);
+	module->stderr_redirect = -1;
 
-	module_conf_dir = g_strdup_printf("%s/modules",
-					  SpeechdOptions.conf_dir);
+	if (module->progdir) {
+		module->filename = (char *)spd_get_path(mod_prog, module->progdir);
+	} else {
+		module->filename = (char *)spd_get_path(mod_prog, SpeechdOptions.user_module_dir);
+		if (stat(module->filename, &fileinfo) != 0) {
+			g_free(module->filename);
+			module->filename = (char *)spd_get_path(mod_prog, SpeechdOptions.module_dir);
+		}
+	}
 
-	module->configfilename =
-	    (char *)spd_get_path(mod_cfgfile, module_conf_dir);
-	g_free(module_conf_dir);
+	if (mod_cfg_dir) {
+		module->configfilename =
+		    (char *)spd_get_path(mod_cfgfile, mod_cfg_dir);
+	} else {
+		module_conf_dir = g_strdup_printf("%s/modules",
+					  SpeechdOptions.user_conf_dir);
+		module->configfilename =
+		    (char *)spd_get_path(mod_cfgfile, module_conf_dir);
+		g_free(module_conf_dir);
+		if (stat(module->configfilename, &fileinfo) != 0) {
+			module_conf_dir = g_strdup_printf("%s/modules",
+							  SpeechdOptions.conf_dir);
+			module->configfilename =
+			    (char *)spd_get_path(mod_cfgfile, module_conf_dir);
+			g_free(module_conf_dir);
+		}
+	}
 
 	if (mod_dbgfile != NULL)
 		module->debugfilename = g_strdup(mod_dbgfile);
@@ -162,6 +410,7 @@ OutputModule *load_output_module(char *mod_name, char *mod_prog,
 	if ((pipe(module->pipe_in) != 0)
 	    || (pipe(module->pipe_out) != 0)) {
 		MSG(3, "Can't open pipe! Module not loaded.");
+		destroy_module(module);
 		return NULL;
 	}
 
@@ -177,7 +426,7 @@ OutputModule *load_output_module(char *mod_name, char *mod_prog,
 					       S_IRUSR | S_IWUSR);
 		if (module->stderr_redirect == -1)
 			MSG(1,
-			    "ERROR: Openning debug file for %s failed: (error=%d) %s",
+			    "ERROR: Opening debug file for %s failed: (error=%d) %s",
 			    module->name, module->stderr_redirect,
 			    strerror(errno));
 	} else {
@@ -197,6 +446,7 @@ OutputModule *load_output_module(char *mod_name, char *mod_prog,
 	fr = fork();
 	if (fr == -1) {
 		printf("Can't fork, error! Module not loaded.");
+		destroy_module(module);
 		return NULL;
 	}
 
@@ -255,32 +505,38 @@ OutputModule *load_output_module(char *mod_name, char *mod_prog,
 		MSG(1, "ERROR: Something wrong with %s, can't initialize",
 		    module->name);
 		output_close(module);
+		destroy_module(module);
 		return NULL;
 	}
 
 	reply = g_string_new("\n---------------\n");
 	f = fdopen(dup(module->pipe_out[0]), "r");
 	while (1) {
-		ret = spd_getline(&rep_line, &n, f);
+		ret = getline(&rep_line, &n, f);
 		if (ret <= 0) {
 			MSG(1, "ERROR: Bad syntax from output module %s 1",
 			    module->name);
-			if (rep_line != NULL)
-				g_free(rep_line);
+			g_string_free(reply, TRUE);
+			free(rep_line);
+			fclose(f);
+			destroy_module(module);
 			return NULL;
 		}
 		assert(rep_line != NULL);
-		MSG(5, "Reply from output module: %d %s", n, rep_line);
+		MSG(5, "Reply from output module: %ld %s", (long) n, rep_line);
 		if (ret <= 4) {
 			MSG(1, "ERROR: Bad syntax from output module %s 2",
 			    module->name);
-			g_free(rep_line);
+			g_string_free(reply, TRUE);
+			free(rep_line);
+			fclose(f);
+			destroy_module(module);
 			return NULL;
 		}
 
 		if (rep_line[3] != '-') {
 			s = rep_line[0];
-			g_free(rep_line);
+			free(rep_line);
 			break;
 		}
 
@@ -337,6 +593,27 @@ OutputModule *load_output_module(char *mod_name, char *mod_prog,
 		return NULL;
 	}
 
+	/* Try to get the list of voices */
+	SPDVoice **voices = output_get_voices(module);
+	if (!voices) {
+		/* No list of voices, that would surprise clients, let's give up
+		 * on this module */
+		MSG(1,
+		    "ERROR: Can't get a list of voices from the output module.");
+		module->working = 0;
+		kill(module->pid, 9);
+		waitpid(module->pid, NULL, WNOHANG);
+		destroy_module(module);
+		return NULL;
+	}
+	for (i = 0; voices[i]; i++) {
+		g_free(voices[i]->name);
+		g_free(voices[i]->language);
+		g_free(voices[i]->variant);
+		g_free(voices[i]);
+	}
+	g_free(voices);
+
 	return module;
 }
 
@@ -375,7 +652,9 @@ int reload_output_module(OutputModule * old_module)
 
 	new_module = load_output_module(old_module->name, old_module->filename,
 					old_module->configfilename,
-					old_module->debugfilename);
+					old_module->debugfilename,
+					old_module->progdir,
+					old_module->configdir);
 	if (new_module == NULL) {
 		MSG(3, "Can't load module %s while reloading modules.",
 		    old_module->name);
@@ -408,6 +687,8 @@ int output_module_debug(OutputModule * module)
 
 	output_send_debug(module, 1, new_log_path);
 
+	g_free(new_log_path);
+
 	return 0;
 }
 
@@ -418,7 +699,7 @@ int output_module_nodebug(OutputModule * module)
 	if (!module->working)
 		return -1;
 
-	MSG(4, "Output module debug logging off for", module->name);
+	MSG(4, "Output module debug logging off for %s", module->name);
 
 	output_send_debug(module, 0, NULL);
 
@@ -485,7 +766,8 @@ module_already_requested(char *module_name, char *module_cmd,
  */
 void
 module_add_load_request(char *module_name, char *module_cmd,
-			char *module_cfgfile, char *module_dbgfile)
+			char *module_cfgfile, char *module_dbgfile,
+			char *module_cmd_dir, char *module_cfg_dir)
 {
 	char **module_params = NULL;
 
@@ -497,17 +779,26 @@ module_add_load_request(char *module_name, char *module_cmd,
 		g_free(module_cmd);
 		g_free(module_cfgfile);
 		g_free(module_dbgfile);
+		g_free(module_cmd_dir);
+		g_free(module_cfg_dir);
 		return;
 	}
 
-	module_params = g_malloc(4 * sizeof(char *));
+	module_params = g_malloc(6 * sizeof(char *));
 	module_params[0] = module_name;
 	module_params[1] = module_cmd;
 	module_params[2] = module_cfgfile;
 	module_params[3] = module_dbgfile;
+	module_params[4] = module_cmd_dir;
+	module_params[5] = module_cfg_dir;
 	requested_modules = g_list_append(requested_modules, module_params);
-	MSG(5, "Module name=%s being inserted into requested_modules list",
-	    module_params[0]);
+	MSG(5, "Module name=%s being inserted into requested_modules list, with '%s' '%s' '%s' '%s' '%s'",
+	    module_params[0],
+	    module_params[1],
+	    module_params[2],
+	    module_params[3],
+	    module_params[4],
+	    module_params[5]);
 }
 
 /*
@@ -524,7 +815,8 @@ void module_load_requested_modules(void)
 
 		new_module =
 		    load_output_module(module_params[0], module_params[1],
-				       module_params[2], module_params[3]);
+				       module_params[2], module_params[3],
+				       module_params[4], module_params[5]);
 
 		if (new_module != NULL)
 			output_modules =
@@ -534,10 +826,17 @@ void module_load_requested_modules(void)
 		g_free(module_params[1]);
 		g_free(module_params[2]);
 		g_free(module_params[3]);
+		g_free(module_params[4]);
+		g_free(module_params[5]);
 		g_free(module_params);
 		requested_modules =
 		    g_list_delete_link(requested_modules, requested_modules);
 	}
+
+	if (output_modules && !GlobalFDSet.output_module) {
+		OutputModule *first_module = output_modules->data;
+		GlobalFDSet.output_module = first_module->name;
+	}
 }
 
 /*
